//
//
//	frames.cpp
//
//	Simple utility to parse a "smil" file (kino edit list) and 
//  return the number of frames therein.  This can be used to check
//  output of encoders to ensure that no frames were dropped, etc.
//
//	*NOTE* Thisis not terribly robust, and depends on the smil format
//	as it is produced by Kino at the time I wrote this.  It is NOT an
//	XML parser.  
//
//	Copyright 2003 by Robert Kesterson, robertk@robertk.com	
//
//	You may freely distribute or use this source code in any
//  way that you choose.  If the unlikely event that you want to
//	 use it in your own program, I would appreciate a mention 
// 	in the credits and/or an email to let me know you 
//	liked it and are using it.
//
//

#include <iostream>
#include <fstream>
#include <string>
#include <stdlib.h>

#define CLIPBEGIN 	"clipBegin=\""
#define CLIPEND 	"clipEnd=\""

void usage(const char *pszProg)
{
   	std::cout << "Usage is: [-t]" << pszProg << " editlist.smil\n";
   	std::cout << "Where editlist.smil is replaced with your edit list file,\n";
   	std::cout << "and '-t' means terse output\n";
}
					   
int main(int argc, char **argv)
{
	if(argc < 2)
	{
		usage(argv[0]);
    	return 10;
	}

	long nFrames = 0;
	bool bTerse = false;
	
	const char *pszFile = argv[1];
	if(argv[1][0] == '-' && argv[0][1] == 't')
	{
		bTerse = true;
		if(argc < 3)
		{
			usage(argv[0]);
			return 10;
		}
		pszFile = argv[2];
	}

	std::ifstream infile;
	infile.open(pszFile);
	if(infile)
	{
		char buffer[1024];
		while(infile.good())
		{
			memset(buffer, 0, sizeof(buffer));
			infile.getline(buffer, sizeof(buffer)-1);
			std::string sIn(buffer);
			int nBeginPos 	= sIn.find(CLIPBEGIN);
			int nEndPos 	= sIn.find(CLIPEND);
			if(nBeginPos != std::string::npos && nEndPos != std::string::npos)
			{
				int nBegin 	= atol(sIn.c_str() + nBeginPos + strlen(CLIPBEGIN));
				int nEnd	= atol(sIn.c_str() + nEndPos + strlen(CLIPEND));
				nFrames += nEnd - nBegin + 1;
			}
		}
		infile.close();
		std::cout << "File references " << nFrames << " frames\n";
	}
	else
	{
		std::cerr << "Unable to read file " << argv[1] << "\n";
		return 10;
	}
	return 0;
}
