//
//
//	smilsplit.cpp
//
//	Simple utility to split a "smil" file (kino edit list) into
//  several smaller chunks, so that encoding can be done on
//	multiple small files instead of one large one (avoids having
//  to redo the whole thing if something goes wrong).
//
//	*NOTE* Thisis not terribly robust, and depends on the smil format
//	as it is produced by Kino at the time I wrote this.  It is NOT an
//	XML parser.  
//
//	Copyright 2003 by Robert Kesterson, robertk@robertk.com	
//
//	You may freely distribute or use this source code in any
//  way that you choose.  If the unlikely event that you want to
//	 use it in your own program, I would appreciate a mention 
// 	in the credits and/or an email to let me know you 
//	liked it and are using it.
//
//

#include <iostream>
#include <fstream>
#include <string>
#include <stdlib.h>
#include <stdio.h>

#define CLIPBEGIN 	"clipBegin=\""
#define CLIPEND 	"clipEnd=\""

void usage(const char *pszProg)
{
   	std::cout << "Usage is: " << pszProg << " editlist.smil num_frames template\n";
   	std::cout << "Where editlist.smil is replaced with your edit list file,\n";
   	std::cout << "num_frames is the number of frames to use per segment, and\n";
   	std::cout << "template is a printf-style file spec template ie MyFile%02d.smil.\n";
}
					   
int main(int argc, char **argv)
{
	if(argc < 4)
	{
		usage(argv[0]);
    	return 10;
	}

	long nFrames = 0;
	
	const char *pszFile = argv[1];
	long num_frames = atol(argv[2]);
	const char *pszTemplate = argv[3];

	std::ifstream infile;
	std::ofstream outfile;
	char szOutName[1024];	

	infile.open(pszFile);
	if(infile)
	{
		int nFileNo = 1;
		char buffer[1024];
		sprintf(szOutName, pszTemplate, nFileNo);
		outfile.open(szOutName);
		while(infile.good() && outfile.good())
		{
			memset(buffer, 0, sizeof(buffer));
			infile.getline(buffer, sizeof(buffer)-1);
			std::string sIn(buffer);
			int nBeginPos 	= sIn.find(CLIPBEGIN);
			int nEndPos 	= sIn.find(CLIPEND);
			if(nBeginPos != std::string::npos && nEndPos != std::string::npos)
			{
				int nBegin 	= atol(sIn.c_str() + nBeginPos + strlen(CLIPBEGIN));
				int nEnd	= atol(sIn.c_str() + nEndPos + strlen(CLIPEND));
				if(nFrames + (nEnd - nBegin + 1) >= num_frames)
				{
					int nSplitFrame = nBegin + (num_frames - nFrames);
					sIn.erase(nEndPos, sIn.length() - nEndPos + 1);
                    outfile << sIn << CLIPEND << nSplitFrame++ << "\" />\n";
					outfile << "  </seq>\n"
						    << "</smil>\n";
					outfile.close();
					sprintf(szOutName, pszTemplate, ++nFileNo);
					outfile.open(szOutName);
					outfile << "<?xml version=\"1.0\"?>\n"
						    << "<smil xmlns:smil2=\"http://www.w3.org/2001/SMIL20/Language\">\n"
							<< "  <seq>\n";
					sIn.erase(nBeginPos, sIn.length() - nBeginPos + 1);
                    outfile << sIn << CLIPBEGIN << nSplitFrame << "\" "
							<< CLIPEND << nEnd << "\" />\n";
					nFrames = 0;
				}
				else
				{
					outfile << buffer << "\n";
					nFrames += nEnd - nBegin + 1;
				}
			}
			else
				outfile << buffer << "\n";
		}
		infile.close();
	}
	else
	{
		std::cerr << "Unable to read file " << argv[1] << "\n";
		return 10;
	}
	return 0;
}
